{ pkgs, nodeName, ... }:
{
  networking.hostName = builtins.elemAt (builtins.split "\\." nodeName) 0; # Define your hostname.
  networking.domain = builtins.elemAt (builtins.split "(^[^\\.]+\.)" nodeName) 2;

  environment.etc."lvm/lvm.conf".text = ''
    devices {
      issue_discards = 1
    }
  '';

  # Fonts, I18N, Date ...
  fonts.packages = [ pkgs.corefonts ];

  console.font = "lat9w-16";

  i18n = {
    defaultLocale = "en_US.UTF-8";
  };
  time.timeZone = "Etc/UTC";
  services.gpm.enable = true;

  services.packagekit.enable = true;
  services.openssh.enable = true;
  networking.firewall.enable = true;

  # Activation scripts for impure set up of paths in /
  system.activationScripts.bin = ''
    echo "setting up /bin..."
    mkdir -p /bin
    ln -sfT ${pkgs.bash}/bin/bash /bin/.bash
    mv -Tf /bin/.bash /bin/bash
  '';

  # TODO: find out if this workaround is still required from nixos 20.03 onwards
  # system.activationScripts.etcX11sessinos = ''
  #   echo "setting up /etc/X11/sessions..."
  #   mkdir -p /etc/X11
  #   ln -sfT ${config.services.xserver.displayManager.session.desktops} /etc/X11/.sessions
  #   mv -Tf /etc/X11/.sessions /etc/X11/sessions
  # '';

  # TODO: adapt this to be arch agnostic
  system.activationScripts.lib64 = ''
    echo "setting up /lib64..."
    mkdir -p /lib64
    ln -sfT ${pkgs.glibc}/lib/ld-linux-x86-64.so.2 /lib64/.ld-linux-x86-64.so.2
    mv -Tf /lib64/.ld-linux-x86-64.so.2 /lib64/ld-linux-x86-64.so.2
  '';
  programs.fuse.userAllowOther = true;
}
